const colours = [
  "e1f6ff",
  "cdf0ff",
  "6ee0ff",
  "a5e5ff",
  "91e0ff",
  "73d7ff",
  "55ceff",
  "37c6ff",
  "19bdff",
  "00abf0",
  "0083b8",
  "03709c",
  "025677",
];

/* chrome_main.js: */

const greyDict = {};

// Invert Color - MIT License (attached)
function invertColor(
  hex,
  bw,
  black = "#000",
  white = "#fff",
  dark = 186,
  back = false,
  bhex = "#505050",
  lowwhite = "a4a4a4"
) {
  if (hex.indexOf("#") === 0) hex = hex.slice(1);
  if (bhex.indexOf("#") === 0) bhex = bhex.slice(1);

  // convert 3-digit hex to 6-digits.
  if (hex.length === 3)
    hex = hex[0] + hex[0] + hex[1] + hex[1] + hex[2] + hex[2];
  if (bhex.length === 3)
    bhex = bhex[0] + bhex[0] + bhex[1] + bhex[1] + bhex[2] + bhex[2];

  if (hex.length !== 6) throw new Error("Invalid HEX color.");
  if (bhex.length !== 6) throw new Error("Invalid BHEX color.");

  let r = parseInt(hex.slice(0, 2), 16),
    g = parseInt(hex.slice(2, 4), 16),
    b = parseInt(hex.slice(4, 6), 16);

  if (bw) return r * 0.299 + g * 0.587 + b * 0.114 > dark ? black : white;

  if (back) {
    // https://snook.ca/technical/colour_contrast/colour
    const br = parseInt(bhex.slice(0, 2), 16),
      bg = parseInt(bhex.slice(2, 4), 16),
      bb = parseInt(bhex.slice(4, 6), 16);
    if (
      Math.abs(
        br * 0.299 +
          bg * 0.587 +
          bb * 0.114 -
          (r * 0.299 + g * 0.587 + b * 0.114)
      ) >= 100 ||
      Math.max(r, br) -
        Math.min(r, br) +
        (Math.max(g, bg) - Math.min(g, bg)) +
        (Math.max(b, bb) - Math.min(b, bb)) >=
        250
    ) {
      return "#" + hex;
    } else {
      if (!greyDict.hasOwnProperty(hex)) {
        greyDict[hex] =
          "#" +
          Math.round(
            Math.random() *
              (parseInt(white.slice(1).slice(0, 2), 16) -
                parseInt(lowwhite.slice(0, 2), 16)) +
              parseInt(lowwhite.slice(0, 2), 16)
          )
            .toString(16)
            .repeat(3);
      }
      return greyDict[hex];
    }
  }
  //
  // invert color components
  r = (255 - r).toString(16);
  g = (255 - g).toString(16);
  b = (255 - b).toString(16);

  // pad each with zeros and return
  return "#" + padZero(r) + padZero(g) + padZero(b);
}

function sleep(ms) {
  return new Promise((resolve) => setTimeout(resolve, ms));
}

function padZero(str, len) {
  len = len || 2;
  return (new Array(len).join("0") + str).slice(-len);
}

function rgbToHex(input) {
  const rgb = input
    .substring(input.indexOf("(") + 1, input.indexOf(")"))
    .split(", ");
  return (
    parseInt(rgb[0]).toString(16).padStart(2, "0") +
    parseInt(rgb[1]).toString(16).padStart(2, "0") +
    parseInt(rgb[2]).toString(16).padStart(2, "0")
  );
}

// Highlighted Text - Sections
function applyHighlightedText(highlighted) {
  for (const lighted of highlighted)
    lighted.style.color = invertColor(rgbToHex(lighted.style.backgroundColor));
}

// Margin on due work
async function fixMargin() {
  while (document.getElementsByClassName("fc-list-heading-main").length === 0)
    await sleep(100);

  for (const heading of document.getElementsByClassName("fc-list-heading-main"))
    heading.style.marginLeft = "10px";
}

async function colourfulText() {
  let highlightedLength = 0;
  let colouredLength = 0;
  for (let i = 0; i < 50; i++) {
    const coloured = document.querySelectorAll(
      "#content article span[style*='color:']:not([style*='background-color:']), li article span[style*='color:']:not([style*='background-color:'])"
    );
    const highlighted = document.querySelectorAll(
      "#content article span[style*='background-color:']"
    );

    if (coloured.length !== colouredLength) {
      applyColouredText(coloured);
      colouredLength = document.querySelectorAll(
        "#content article span[style*='color:']:not([style*='background-color:']), li article span[style*='color:']:not([style*='background-color:'])"
      ).length;
    }

    if (highlighted.length !== highlightedLength) {
      for (let high of highlighted)
        if (
          high.childElementCount === 1 &&
          high.style.color === "rgb(0, 0, 0)" &&
          high.style.backgroundColor === "rgb(255, 255, 255)"
        ) {
          high.style.color = "inherit";
          high.style.backgroundColor = "inherit";
          high.children[0].style.color = "inherit";
        }

      applyHighlightedText(highlighted);
      highlightedLength = document.querySelectorAll(
        "#content article span[style*='background-color:']"
      ).length;
    }

    await sleep(100);
  }
}

const spantimes = [];
const thistimes = [];
const subjtimes = [];
const timetableSubjects = [];
const myHeadings = [];
const onentimes = [];
const toolkitParentPlanner = [];

const subjdic = {};

function read() {
  // Due Times
  const spanx = document.evaluate(
    "//span[contains(text(),'Due')]/time/..",
    document,
    null,
    XPathResult.ANY_TYPE,
    null
  );
  let currentspan;

  while ((currentspan = spanx.iterateNext())) {
    spantimes.push(currentspan);
  }

  // This Unit
  // OLD (works but slow): //div[@class='list-item']/div[@class='card small-12']/h3/a[contains(concat(' ',normalize-space(@class),' '),' expand-link ')]/../../p[contains(text(),'This unit')]
  const thisx = document.evaluate(
    "//div[@class='card small-12']/p[contains(text(),'This unit')]",
    document,
    null,
    XPathResult.ANY_TYPE,
    null
  );
  let currentthis;

  while ((currentthis = thisx.iterateNext())) {
    thistimes.push(currentthis);
  }

  // Timetable Subjects
  const subjx = document.evaluate(
    '//div[@class="timetable-subject"]/../../../th[not(contains(text(),"Co-Curricular"))]/..//div[@class="timetable-subject"] | //div[@class="timetable-subject"]/../../../../tr/td/div/div[contains(@class, "timetable-subject")]/*[@class=""]/..',
    document,
    null,
    XPathResult.ANY_TYPE,
    null
  );
  let currentsubj;

  while ((currentsubj = subjx.iterateNext())) {
    subjtimes.push(currentsubj);
  }

  let count = 0;
  for (const subj of subjtimes) {
    let subjtext;
    if (subj.children[0].nodeName === "A")
      subjtext = subj.children[0].textContent;
    else subjtext = subj.childNodes[0].textContent;

    if (!(subjtext in subjdic) && subjtext.trim() !== "") {
      count++;
      subjdic[subjtext] = Math.min(13, count + 1);
    }
  }

  for (const subj in subjdic) {
    const lesson = document.evaluate(
      '//div[@class="timetable-subject"]/a[text()="' +
        subj +
        '"]/..|//div[@class="timetable-subject"][text()="' +
        subj +
        '"]',
      document,
      null,
      XPathResult.ANY_TYPE,
      null
    );

    const colourpairs = [];
    const invColor = invertColor(
      colours[subjdic[subj] - 1],
      true,
      "#000",
      "#fff",
      140
    );

    let currentless;
    while ((currentless = lesson.iterateNext())) {
      colourpairs.push([
        currentless,
        currentless.style.color,
        currentless.style.backgroundColor,
      ]);
    }

    timetableSubjects.push([colourpairs, subjdic[subj], invColor]);
  }

  // Table Text - Fix Colours
  for (const tabletd of document.querySelectorAll(
    "#content article td[style*='background-color:']"
  )) {
    tabletd.style.color = invertColor(
      rgbToHex(tabletd.style.backgroundColor),
      true
    );
  }

  // OneNote Iframes
  const onenotex = document.evaluate(
    "//h2[@title='OneNote']/../../../../../..//iframe",
    document,
    null,
    XPathResult.ANY_TYPE,
    null
  );
  let currentone;

  while ((currentone = onenotex.iterateNext())) {
    onentimes.push(currentone);
  }

  // Make 'my favourite groups' and 'my classes' render properly on hover
  let subjectHeading, groupHeading;
  if (
    (subjectHeading = document.getElementById("side-menu-mysubjects")) !== null
  )
    myHeadings.push(subjectHeading);
  if ((groupHeading = document.getElementById("side-menu-mygroups")) !== null)
    myHeadings.push(groupHeading);

  // Calculate toolkit and planner positions for moving
  const toolkit = document.getElementsByClassName(
    "Schoolbox_Learning_Moment_StudentToolkit_Component_Dashboard_Controller"
  );

  if (toolkit.length > 0) {
    toolkitParentPlanner[0] = toolkit[0];

    // Middle school seems to use tileList 6001
    let planner = document.getElementById("tileList-6001");
    if (planner === null) {
      // Senior school seems to use tileList 2791
      planner = document.getElementById("tileList-2791");
      if (planner !== null) {
        toolkitParentPlanner[1] = toolkit[0].parentElement;
        toolkitParentPlanner[2] =
          planner.parentElement.parentElement.parentElement.parentElement;
      }
    } else {
      toolkitParentPlanner[1] = toolkit[0].parentElement;
      toolkitParentPlanner[2] =
        planner.parentElement.parentElement.parentElement.parentElement;
    }
  }
}

// Grades colour fix
const gradenum = {
  "A+": 12,
  Outstanding: 12,
  A: 11,
  "A-": 10,
  Good: 10,
  "B+": 9,
  B: 8,
  Satisfactory: 8,
  "B-": 7,
  "C+": 6,
  Inconsistent: 6,
  C: 5,
  "C-": 4,
  "Not Satisfactory": 4,
  "D+": 3,
  D: 2,
  "D-": 2,
  "E+": 2,
  E: 2,
  "E-": 2,
  Absent: 1,
};

const percgrades = {
  101: "A+",
  95: "A+",
  89: "A",
  83: "A-",
  77: "B+",
  71: "B",
  65: "B-",
  59: "C+",
  53: "C",
  47: "C-",
  41: "D+",
  35: "D",
  29: "D-",
  23: "E+",
  17: "E",
  0: "E-",
};

function schoolboxDarkMode() {
  /* read first */
  read();

  /* START WRITING BABEE */

  const gradeNodes = document.getElementsByClassName("grade");
  for (const gradeNode of gradeNodes) {
    let grade = "0";
    const gradeStr = gradeNode.children[0].textContent;

    if (Object.keys(gradenum).includes(gradeStr)) grade = gradenum[gradeStr];
    else if (!isNaN((percentage = parseInt(gradeStr)))) {
      const gradePercentages = Object.keys(percgrades);
      for (const percg of gradePercentages) {
        if (percentage < percg) {
          grade =
            gradenum[
              Object.values(percgrades)[gradePercentages.indexOf(percg) - 1]
            ];
          break;
        }
      }
    }

    if (grade == 0) continue;

    for (const a of gradeNode.classList)
      if (a.substring(0, 5) === "gradi") gradeNode.classList.remove(a);

    gradeNode.style.backgroundColor = "var(--color-" + grade.toString() + ")";
    gradeNode.style.color = invertColor(
      colours[grade - 1],
      true,
      "var(--body-background)",
      "var(--body-foreground)",
      200
    );
  }

  let activeSubjectColour = null;
  for (const subject of timetableSubjects)
    for (const arr of subject[0]) {
      if (arr[2].indexOf("var") === -1)
        arr[0].style.background = "var(--color-" + subject[1] + ")";
      if (arr[1].indexOf("var") === -1) arr[0].style.color = subject[2];

      if (arr[0].parentElement.classList.contains("timetable-subject-active"))
        activeSubjectColour = "var(--color-" + subject[1] + ")";

      arr[0].children[0].style.color = subject[2];
    }

  // Current subject
  if (
    document.getElementsByClassName("timetable-period-active").length > 0 &&
    activeSubjectColour !== null
  )
    document
      .getElementsByClassName("timetable-period-active")[0]
      .setAttribute("style", "color: " + activeSubjectColour);
  if (
    document.getElementsByClassName("timetable-day-active").length > 0 &&
    activeSubjectColour !== null
  )
    document
      .getElementsByClassName("timetable-day-active")[0]
      .setAttribute("style", "color: " + activeSubjectColour);

  for (const onen of onentimes) onen.style.filter = "invert(100%);";

  for (const heading of myHeadings)
    if (heading.childElementCount > 0)
      heading.firstElementChild.classList += "side-menu-heading-link";

  colourfulText();
  fixMargin();

  // Add Schoolbox Dark Mode in side menu
  const modeSpan = document.createElement("span");
  modeSpan.textContent = "DARK MODE WEBSITE";
  const modeIcon = document.createElement("a");
  modeIcon.classList = "icon-dmwebsite";
  modeIcon.href = "https://schoolbox-darkmode.web.app";
  modeIcon.target = "_blank";
  modeIcon.title = "Go to the Schoolbox Darkmode Website";
  modeIcon.appendChild(modeSpan);
  const modeListItem = document.createElement("li");
  modeListItem.setAttribute("data-id", "90");
  modeListItem.setAttribute("style", "display: list-item;");
  modeListItem.appendChild(modeIcon);
  if (document.getElementById("side-menu"))
    document.getElementById("side-menu").appendChild(modeListItem);

  // Add Schoolbox planner above 'toolkit'
  if (toolkitParentPlanner.length > 0) {
    toolkitParentPlanner[1].insertBefore(
      toolkitParentPlanner[2],
      toolkitParentPlanner[0]
    );
  }

  /* :chrome_main.js */

  // Coloured Text - dark
  function applyColouredText(coloured) {
    for (const colour of coloured) {
      if (
        !Array.from(
          document.querySelectorAll("#content article table span")
        ).includes(colour) &&
        colour.parentElement.nodeName !== "SPAN" &&
        colour.parentElement.parentElement.nodeName !== "SPAN"
      ) {
        const invColour = invertColor(
          rgbToHex(colour.style.color),
          false,
          "#000",
          "#fff",
          186,
          true
        );

        if (colour.childElementCount !== 0) {
          for (const i of colour.children)
            if (i.nodeName !== "STRONG") colour.style.color = invColour;
        } else colour.style.color = invColour;
      }
    }
  }

  // Group colours - dark
  for (const elem of document.querySelectorAll("div.card p.meta")) {
    if (elem.textContent === "TCS Blue Group Student")
      elem.style.color = "#0064ff";
    else if (elem.textContent === "TCS Red Group Student")
      elem.style.color = "red";
    else if (elem.textContent === "Junior School Students")
      elem.style.color = "var(--color-1)";
    else if (elem.textContent === "Middle School Students")
      elem.style.color = "var(--color-2)";
    else if (elem.textContent === "Teachers")
      elem.style.color = "var(--color-3)";
    else if (elem.textContent === "Pre-service Teachers")
      elem.style.color = "var(--color-4)";
    else if (elem.textContent === "Non-Teaching Staff")
      elem.style.color = "var(--color-9)";
    else if (elem.textContent === "TCS Yellow Group Student") {
      elem.innerText = "TCS Gold Group Student";
      elem.style.color = "#d4af37";
    }
  }

  /* Dark mode only */
  for (const span of spantimes)
    span.setAttribute("style", "font-weight: 700; filter: brightness(150%);");

  for (const unit of thistimes)
    unit.setAttribute("style", "font-weight: 700; filter: brightness(150%);");

  // New Logo - Chrome Special
  for (let logo of document.querySelectorAll(
    ".logo img:not(.tab-bar-section)"
  )) {
    logo.srcset = `${chrome.runtime.getURL("logo.png")} 2x`;
    logo.removeAttribute("src");
    logo.setAttribute("updated", "true");
  }

  let logo = document.querySelector(".tab-bar-section .logo img");
  if (logo != null) {
    logo.srcset = `${chrome.runtime.getURL("small_logo.png")} 1x`;
    logo.removeAttribute("src");
    logo.setAttribute("updated", "true");
  }

  document.querySelector("link[rel='Shortcut Icon']").href =
    chrome.runtime.getURL("favicon.png");
  if (document.querySelector(".error-state .row img, .empty-state img"))
    document.querySelector(".error-state .row img, .empty-state img").src =
      chrome.runtime.getURL("error.png");
  if (document.querySelector("body.login.side")) {
    let bodyLogin = document.querySelector("body.login.side");
    bodyLogin.style.backgroundImage = `url(${chrome.runtime.getURL(
      "login.png"
    )})`;
    bodyLogin.setAttribute("updated", "true");
  }
  if (
    document.querySelector(
      "div.cover-photo[data-role='eportfolio-cover-photo']"
    )
  ) {
    let eportfolio = document.querySelector(
      "div.cover-photo[data-role='eportfolio-cover-photo']"
    );
    if (eportfolio.style.backgroundImage == "")
      eportfolio.style.backgroundImage = `url(${chrome.runtime.getURL(
        "login.png"
      )})`;
    eportfolio.setAttribute("updated", "true");
  }

  // Darkmode Font - Chrome Special
  let font = document.createElement("style");
  font.type = "text/css";
  font.appendChild(
    document.createTextNode(
      `@font-face { font-family: 'darkmode'; src: url('${chrome.runtime.getURL(
        "darkmode.otf"
      )}') format('opentype'); font-weight: normal; font-style: normal; }`
    )
  );

  document.head.appendChild(font);
}

window.addEventListener("DOMContentLoaded", schoolboxDarkMode);
